from typing import Any

class TraversibleType(type):
    def __init__(cls, clsname, bases, clsdict) -> None: ...

class Traversible:
    def __class_getitem__(cls, key): ...
    def get_children(self, omit_attrs=..., **kw): ...

class _InternalTraversalType(type):
    def __init__(cls, clsname, bases, clsdict) -> None: ...

class InternalTraversal:
    def dispatch(self, visit_symbol): ...
    def run_generated_dispatch(self, target, internal_dispatch, generate_dispatcher_name): ...
    def generate_dispatch(self, target_cls, internal_dispatch, generate_dispatcher_name): ...
    dp_has_cache_key: Any
    dp_has_cache_key_list: Any
    dp_clauseelement: Any
    dp_fromclause_canonical_column_collection: Any
    dp_clauseelement_tuples: Any
    dp_clauseelement_list: Any
    dp_clauseelement_tuple: Any
    dp_executable_options: Any
    dp_with_context_options: Any
    dp_fromclause_ordered_set: Any
    dp_string: Any
    dp_string_list: Any
    dp_anon_name: Any
    dp_boolean: Any
    dp_operator: Any
    dp_type: Any
    dp_plain_dict: Any
    dp_dialect_options: Any
    dp_string_clauseelement_dict: Any
    dp_string_multi_dict: Any
    dp_annotations_key: Any
    dp_plain_obj: Any
    dp_named_ddl_element: Any
    dp_prefix_sequence: Any
    dp_table_hint_list: Any
    dp_setup_join_tuple: Any
    dp_memoized_select_entities: Any
    dp_statement_hint_list: Any
    dp_unknown_structure: Any
    dp_dml_ordered_values: Any
    dp_dml_values: Any
    dp_dml_multi_values: Any
    dp_propagate_attrs: Any

class ExtendedInternalTraversal(InternalTraversal):
    dp_ignore: Any
    dp_inspectable: Any
    dp_multi: Any
    dp_multi_list: Any
    dp_has_cache_key_tuples: Any
    dp_inspectable_list: Any

class ExternalTraversal:
    __traverse_options__: Any
    def traverse_single(self, obj, **kw): ...
    def iterate(self, obj): ...
    def traverse(self, obj): ...
    @property
    def visitor_iterator(self) -> None: ...
    def chain(self, visitor): ...

class CloningExternalTraversal(ExternalTraversal):
    def copy_and_process(self, list_): ...
    def traverse(self, obj): ...

class ReplacingExternalTraversal(CloningExternalTraversal):
    def replace(self, elem) -> None: ...
    def traverse(self, obj): ...

Visitable = Traversible
VisitableType = TraversibleType
ClauseVisitor = ExternalTraversal
CloningVisitor = CloningExternalTraversal
ReplacingCloningVisitor = ReplacingExternalTraversal

def iterate(obj, opts=...) -> None: ...
def traverse_using(iterator, obj, visitors): ...
def traverse(obj, opts, visitors): ...
def cloned_traverse(obj, opts, visitors): ...
def replacement_traverse(obj, opts, replace): ...



from typing import Any

from ...connectors.pyodbc import PyODBCConnector
from .base import MySQLDialect, MySQLExecutionContext
from .types import TIME

class _pyodbcTIME(TIME):
    def result_processor(self, dialect, coltype): ...

class MySQLExecutionContext_pyodbc(MySQLExecutionContext):
    def get_lastrowid(self): ...

class MySQLDialect_pyodbc(PyODBCConnector, MySQLDialect):
    supports_statement_cache: bool
    colspecs: Any
    supports_unicode_statements: bool
    pyodbc_driver_name: str
    def on_connect(self): ...

dialect = MySQLDialect_pyodbc



from typing import Any

import sqlalchemy.types as sqltypes

from ...engine import default
from ...sql import compiler
from ...types import (
    BLOB as BLOB,
    BOOLEAN as BOOLEAN,
    CHAR as CHAR,
    DECIMAL as DECIMAL,
    FLOAT as FLOAT,
    INTEGER as INTEGER,
    NUMERIC as NUMERIC,
    REAL as REAL,
    SMALLINT as SMALLINT,
    TEXT as TEXT,
    TIMESTAMP as TIMESTAMP,
    VARCHAR as VARCHAR,
)
from .json import JSON as JSON

class _SQliteJson(JSON):
    def result_processor(self, dialect, coltype): ...

class _DateTimeMixin:
    def __init__(self, storage_format: Any | None = ..., regexp: Any | None = ..., **kw) -> None: ...
    @property
    def format_is_text_affinity(self): ...
    def adapt(self, cls, **kw): ...
    def literal_processor(self, dialect): ...

class DATETIME(_DateTimeMixin, sqltypes.DateTime):
    def __init__(self, *args, **kwargs) -> None: ...
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class DATE(_DateTimeMixin, sqltypes.Date):
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class TIME(_DateTimeMixin, sqltypes.Time):
    def __init__(self, *args, **kwargs) -> None: ...
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

colspecs: Any
ischema_names: Any

class SQLiteCompiler(compiler.SQLCompiler):
    extract_map: Any
    def visit_now_func(self, fn, **kw): ...
    def visit_localtimestamp_func(self, func, **kw): ...
    def visit_true(self, expr, **kw): ...
    def visit_false(self, expr, **kw): ...
    def visit_char_length_func(self, fn, **kw): ...
    def visit_cast(self, cast, **kwargs): ...
    def visit_extract(self, extract, **kw): ...
    def limit_clause(self, select, **kw): ...
    def for_update_clause(self, select, **kw): ...
    def visit_is_distinct_from_binary(self, binary, operator, **kw): ...
    def visit_is_not_distinct_from_binary(self, binary, operator, **kw): ...
    def visit_json_getitem_op_binary(self, binary, operator, **kw): ...
    def visit_json_path_getitem_op_binary(self, binary, operator, **kw): ...
    def visit_empty_set_op_expr(self, type_, expand_op): ...
    def visit_empty_set_expr(self, element_types): ...
    def visit_regexp_match_op_binary(self, binary, operator, **kw): ...
    def visit_not_regexp_match_op_binary(self, binary, operator, **kw): ...
    def visit_on_conflict_do_nothing(self, on_conflict, **kw): ...
    def visit_on_conflict_do_update(self, on_conflict, **kw): ...

class SQLiteDDLCompiler(compiler.DDLCompiler):
    def get_column_specification(self, column, **kwargs): ...
    def visit_primary_key_constraint(self, constraint): ...
    def visit_unique_constraint(self, constraint): ...
    def visit_check_constraint(self, constraint): ...
    def visit_column_check_constraint(self, constraint): ...
    def visit_foreign_key_constraint(self, constraint): ...
    def define_constraint_remote_table(self, constraint, table, preparer): ...
    def visit_create_index(self, create, include_schema: bool = ..., include_table_schema: bool = ...): ...  # type: ignore[override]
    def post_create_table(self, table): ...

class SQLiteTypeCompiler(compiler.GenericTypeCompiler):
    def visit_large_binary(self, type_, **kw): ...
    def visit_DATETIME(self, type_, **kw): ...
    def visit_DATE(self, type_, **kw): ...
    def visit_TIME(self, type_, **kw): ...
    def visit_JSON(self, type_, **kw): ...

class SQLiteIdentifierPreparer(compiler.IdentifierPreparer):
    reserved_words: Any

class SQLiteExecutionContext(default.DefaultExecutionContext): ...

class SQLiteDialect(default.DefaultDialect):
    name: str
    supports_alter: bool
    supports_unicode_statements: bool
    supports_unicode_binds: bool
    supports_default_values: bool
    supports_default_metavalue: bool
    supports_empty_insert: bool
    supports_cast: bool
    supports_multivalues_insert: bool
    tuple_in_values: bool
    supports_statement_cache: bool
    default_paramstyle: str
    statement_compiler: Any
    ddl_compiler: Any
    type_compiler: Any
    preparer: Any
    ischema_names: Any
    colspecs: Any
    isolation_level: Any
    construct_arguments: Any
    native_datetime: Any
    def __init__(
        self,
        isolation_level: Any | None = ...,
        native_datetime: bool = ...,
        json_serializer: Any | None = ...,
        json_deserializer: Any | None = ...,
        _json_serializer: Any | None = ...,
        _json_deserializer: Any | None = ...,
        **kwargs,
    ) -> None: ...
    def set_isolation_level(self, connection, level) -> None: ...
    def get_isolation_level(self, connection): ...
    def on_connect(self): ...
    def get_schema_names(self, connection, **kw): ...
    def get_table_names(self, connection, schema: Any | None = ..., **kw): ...
    def get_temp_table_names(self, connection, **kw): ...
    def get_temp_view_names(self, connection, **kw): ...
    def has_table(self, connection, table_name, schema: Any | None = ...): ...  # type: ignore[override]
    def get_view_names(self, connection, schema: Any | None = ..., **kw): ...
    def get_view_definition(self, connection, view_name, schema: Any | None = ..., **kw): ...
    def get_columns(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_pk_constraint(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_foreign_keys(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_unique_constraints(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_check_constraints(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_indexes(self, connection, table_name, schema: Any | None = ..., **kw): ...



class BasicEntity:
    def __init__(self, **kw) -> None: ...

class ComparableMixin:
    def __ne__(self, other): ...
    def __eq__(self, other): ...

class ComparableEntity(ComparableMixin, BasicEntity):
    def __hash__(self) -> int: ...



from typing import Any

class _MapperConfig:
    @classmethod
    def setup_mapping(cls, registry, cls_, dict_, table, mapper_kw): ...
    cls: Any
    classname: Any
    properties: Any
    declared_attr_reg: Any
    def __init__(self, registry, cls_, mapper_kw) -> None: ...
    def set_cls_attribute(self, attrname, value): ...

class _ImperativeMapperConfig(_MapperConfig):
    dict_: Any
    local_table: Any
    inherits: Any
    def __init__(self, registry, cls_, table, mapper_kw) -> None: ...
    def map(self, mapper_kw=...): ...

class _ClassScanMapperConfig(_MapperConfig):
    dict_: Any
    local_table: Any
    persist_selectable: Any
    declared_columns: Any
    column_copies: Any
    table_args: Any
    tablename: Any
    mapper_args: Any
    mapper_args_fn: Any
    inherits: Any
    def __init__(self, registry, cls_, dict_, table, mapper_kw) -> None: ...
    def map(self, mapper_kw=...): ...

class _DeferredMapperConfig(_ClassScanMapperConfig):
    @property
    def cls(self): ...
    @cls.setter
    def cls(self, class_) -> None: ...
    @classmethod
    def has_cls(cls, class_): ...
    @classmethod
    def raise_unmapped_for_cls(cls, class_) -> None: ...
    @classmethod
    def config_for_cls(cls, class_): ...
    @classmethod
    def classes_for_base(cls, base_cls, sort: bool = ...): ...
    def map(self, mapper_kw=...): ...



from collections.abc import Iterable, Iterator
from typing import Any, TypeVar, overload
from typing_extensions import TypeAlias

_CallExpr: TypeAlias = Any  # mypy.nodes._CallExpr
_Context: TypeAlias = Any  # mypy.nodes._Context
_Expression: TypeAlias = Any  # mypy.nodes._Expression
_JsonDict: TypeAlias = Any  # mypy.nodes._JsonDict
_NameExpr: TypeAlias = Any  # mypy.nodes._NameExpr
_Statement: TypeAlias = Any  # mypy.nodes._Statement
_TypeInfo: TypeAlias = Any  # mypy.nodes._TypeInfo
_ClassDefContext: TypeAlias = Any  # mypy.plugin._ClassDefContext
_DynamicClassDefContext: TypeAlias = Any  # mypy.plugin._DynamicClassDefContext
_SemanticAnalyzerPluginInterface: TypeAlias = Any  # mypy.plugin._SemanticAnalyzerPluginInterface
_Type: TypeAlias = Any  # mypy.types._Type

_TArgType = TypeVar("_TArgType", bound=_CallExpr | _NameExpr)

class SQLAlchemyAttribute:
    name: Any
    line: Any
    column: Any
    type: Any
    info: Any
    def __init__(self, name: str, line: int, column: int, typ: _Type | None, info: _TypeInfo) -> None: ...
    def serialize(self) -> _JsonDict: ...
    def expand_typevar_from_subtype(self, sub_type: _TypeInfo) -> None: ...
    @classmethod
    def deserialize(cls, info: _TypeInfo, data: _JsonDict, api: _SemanticAnalyzerPluginInterface) -> SQLAlchemyAttribute: ...

def name_is_dunder(name): ...
def establish_as_sqlalchemy(info: _TypeInfo) -> None: ...
def set_is_base(info: _TypeInfo) -> None: ...
def get_is_base(info: _TypeInfo) -> bool: ...
def has_declarative_base(info: _TypeInfo) -> bool: ...
def set_has_table(info: _TypeInfo) -> None: ...
def get_has_table(info: _TypeInfo) -> bool: ...
def get_mapped_attributes(info: _TypeInfo, api: _SemanticAnalyzerPluginInterface) -> list[SQLAlchemyAttribute] | None: ...
def set_mapped_attributes(info: _TypeInfo, attributes: list[SQLAlchemyAttribute]) -> None: ...
def fail(api: _SemanticAnalyzerPluginInterface, msg: str, ctx: _Context) -> None: ...
def add_global(ctx: _ClassDefContext | _DynamicClassDefContext, module: str, symbol_name: str, asname: str) -> None: ...
@overload
def get_callexpr_kwarg(callexpr: _CallExpr, name: str, *, expr_types: None = ...) -> _CallExpr | _NameExpr | None: ...
@overload
def get_callexpr_kwarg(callexpr: _CallExpr, name: str, *, expr_types: tuple[type[_TArgType], ...]) -> _TArgType | None: ...
def flatten_typechecking(stmts: Iterable[_Statement]) -> Iterator[_Statement]: ...
def unbound_to_instance(api: _SemanticAnalyzerPluginInterface, typ: _Type) -> _Type: ...
def info_for_cls(cls, api: _SemanticAnalyzerPluginInterface) -> _TypeInfo | None: ...
def expr_to_mapped_constructor(expr: _Expression) -> _CallExpr: ...



from typing import Any

class _ModuleRegistry:
    module_registry: Any
    prefix: Any
    def __init__(self, prefix: str = ...) -> None: ...
    def preload_module(self, *deps): ...
    def import_prefix(self, path) -> None: ...

preloaded: Any
preload_module: Any



import sqlalchemy.types as sqltypes

class JSON(sqltypes.JSON): ...

class _FormatTypeMixin:
    def bind_processor(self, dialect): ...
    def literal_processor(self, dialect): ...

class JSONIndexType(_FormatTypeMixin, sqltypes.JSON.JSONIndexType): ...
class JSONPathType(_FormatTypeMixin, sqltypes.JSON.JSONPathType): ...



from collections.abc import Callable
from typing import Any, ClassVar, TypeVar, overload
from typing_extensions import TypeAlias

from ..engine.interfaces import Connectable
from ..sql.schema import MetaData
from ..util import hybridproperty
from . import interfaces

_ClsT = TypeVar("_ClsT", bound=type[Any])
_DeclT = TypeVar("_DeclT", bound=type[_DeclarativeBase])

# Dynamic class as created by registry.generate_base() via DeclarativeMeta
# or another metaclass. This class does not exist at runtime.
class _DeclarativeBase(Any):  # super classes are dynamic
    registry: ClassVar[registry]
    metadata: ClassVar[MetaData]
    __abstract__: ClassVar[bool]
    # not always existing:
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    __mapper_cls__: ClassVar[Any]
    __class_getitem__: ClassVar[Any]

# Meta class (or function) that creates a _DeclarativeBase class.
_DeclarativeBaseMeta: TypeAlias = Callable[[str, tuple[type[Any], ...], dict[str, Any]], _DeclT]

def has_inherited_table(cls: type[Any]) -> bool: ...

class DeclarativeMeta(type):
    def __init__(cls, classname: str, bases: tuple[type[Any], ...], dict_: dict[str, Any], **kw: object) -> None: ...
    def __setattr__(cls, key: str, value: Any) -> None: ...
    def __delattr__(cls, key: str) -> None: ...

def synonym_for(name, map_column: bool = ...): ...

class declared_attr(interfaces._MappedAttribute, property):
    def __init__(self, fget, cascading: bool = ...) -> None: ...
    def __get__(self, self_, cls): ...
    @hybridproperty
    def cascading(self): ...

class _stateful_declared_attr(declared_attr):
    kw: Any
    def __init__(self, **kw) -> None: ...
    def __call__(self, fn): ...

def declarative_mixin(cls: _ClsT) -> _ClsT: ...
@overload
def declarative_base(
    bind: Connectable | None = ...,
    metadata: MetaData | None = ...,
    mapper: Any | None = ...,
    cls: type[Any] | tuple[type[Any], ...] = ...,
    name: str = ...,
    constructor: Callable[..., None] = ...,
    class_registry: dict[str, type[Any]] | None = ...,
) -> type[_DeclarativeBase]: ...
@overload
def declarative_base(
    bind: Connectable | None = ...,
    metadata: MetaData | None = ...,
    mapper: Any | None = ...,
    cls: type[Any] | tuple[type[Any], ...] = ...,
    name: str = ...,
    constructor: Callable[..., None] = ...,
    class_registry: dict[str, type[Any]] | None = ...,
    *,
    metaclass: _DeclarativeBaseMeta[_DeclT],
) -> _DeclT: ...
@overload
def declarative_base(
    bind: Connectable | None,
    metadata: MetaData | None,
    mapper: Any | None,
    cls: type[Any] | tuple[type[Any], ...],
    name: str,
    constructor: Callable[..., None],
    class_registry: dict[str, type[Any]] | None,
    metaclass: _DeclarativeBaseMeta[_DeclT],
) -> _DeclT: ...

class registry:
    metadata: MetaData
    constructor: Callable[..., None]
    def __init__(
        self,
        metadata: MetaData | None = ...,
        class_registry: dict[str, type[Any]] | None = ...,
        constructor: Callable[..., None] = ...,
        _bind: Connectable | None = ...,
    ) -> None: ...
    @property
    def mappers(self) -> frozenset[Any]: ...
    def configure(self, cascade: bool = ...) -> None: ...
    def dispose(self, cascade: bool = ...) -> None: ...
    @overload
    def generate_base(
        self, mapper: Any | None = ..., cls: type[Any] | tuple[type[Any], ...] = ..., name: str = ...
    ) -> type[_DeclarativeBase]: ...
    @overload
    def generate_base(
        self,
        mapper: Any | None = ...,
        cls: type[Any] | tuple[type[Any], ...] = ...,
        name: str = ...,
        *,
        metaclass: _DeclarativeBaseMeta[_DeclT],
    ) -> _DeclT: ...
    @overload
    def generate_base(
        self, mapper: Any | None, cls: type[Any] | tuple[type[Any], ...], name: str, metaclass: _DeclarativeBaseMeta[_DeclT]
    ) -> type[_DeclarativeBase]: ...
    def mapped(self, cls: _ClsT) -> _ClsT: ...
    # Return type of the callable is a _DeclarativeBase class with the passed in class as base.
    # This could be better approximated with Intersection[PassedInClass, _DeclarativeBase].
    @overload
    def as_declarative_base(self, *, mapper: Any | None = ...) -> Callable[[_ClsT], _ClsT | DeclarativeMeta | Any]: ...
    @overload
    def as_declarative_base(
        self, *, mapper: Any | None = ..., metaclass: _DeclarativeBaseMeta[_DeclT]
    ) -> Callable[[_ClsT], _ClsT | _DeclT | Any]: ...
    def map_declaratively(self, cls): ...
    def map_imperatively(self, class_, local_table: Any | None = ..., **kw): ...

@overload
def as_declarative(
    *,
    bind: Connectable | None = ...,
    metadata: MetaData | None = ...,
    class_registry: dict[str, type[Any]] | None = ...,
    mapper: Any | None = ...,
) -> Callable[[_ClsT], _ClsT | DeclarativeMeta | Any]: ...
@overload
def as_declarative(
    *,
    bind: Connectable | None = ...,
    metadata: MetaData | None = ...,
    class_registry: dict[str, type[Any]] | None = ...,
    mapper: Any | None = ...,
    metaclass: _DeclarativeBaseMeta[_DeclT],
) -> Callable[[_ClsT], _ClsT | _DeclT | Any]: ...



from typing import Any

class ProfileStatsFile:
    force_write: Any
    write: Any
    fname: Any
    short_fname: Any
    data: Any
    dump: Any
    sort: Any
    def __init__(self, filename, sort: str = ..., dump: Any | None = ...): ...
    @property
    def platform_key(self): ...
    def has_stats(self): ...
    def result(self, callcount): ...
    def reset_count(self) -> None: ...
    def replace(self, callcount) -> None: ...

def function_call_count(variance: float = ..., times: int = ..., warmup: int = ...): ...
def count_functions(variance: float = ...) -> None: ...



from typing import Any

import sqlalchemy.types as sqltypes

from .array import ARRAY as PGARRAY
from .base import ENUM, UUID, PGCompiler, PGDialect, PGExecutionContext, PGIdentifierPreparer
from .hstore import HSTORE
from .json import JSON, JSONB

logger: Any

class _PGNumeric(sqltypes.Numeric):
    def bind_processor(self, dialect) -> None: ...
    def result_processor(self, dialect, coltype): ...

class _PGEnum(ENUM):
    def result_processor(self, dialect, coltype): ...

class _PGHStore(HSTORE):
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class _PGARRAY(PGARRAY):
    def bind_expression(self, bindvalue): ...

class _PGJSON(JSON):
    def result_processor(self, dialect, coltype) -> None: ...

class _PGJSONB(JSONB):
    def result_processor(self, dialect, coltype) -> None: ...

class _PGUUID(UUID):
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class PGExecutionContext_psycopg2(PGExecutionContext):
    def create_server_side_cursor(self): ...
    cursor_fetch_strategy: Any
    def post_exec(self) -> None: ...

class PGCompiler_psycopg2(PGCompiler): ...
class PGIdentifierPreparer_psycopg2(PGIdentifierPreparer): ...

EXECUTEMANY_PLAIN: Any
EXECUTEMANY_BATCH: Any
EXECUTEMANY_VALUES: Any
EXECUTEMANY_VALUES_PLUS_BATCH: Any

class PGDialect_psycopg2(PGDialect):
    driver: str
    supports_statement_cache: bool
    supports_unicode_statements: bool
    supports_server_side_cursors: bool
    default_paramstyle: str
    supports_sane_multi_rowcount: bool
    statement_compiler: Any
    preparer: Any
    psycopg2_version: Any
    engine_config_types: Any
    colspecs: Any
    use_native_unicode: Any
    use_native_hstore: Any
    use_native_uuid: Any
    supports_unicode_binds: Any
    client_encoding: Any
    executemany_mode: Any
    insert_executemany_returning: bool
    executemany_batch_page_size: Any
    executemany_values_page_size: Any
    def __init__(
        self,
        use_native_unicode: bool = ...,
        client_encoding: Any | None = ...,
        use_native_hstore: bool = ...,
        use_native_uuid: bool = ...,
        executemany_mode: str = ...,
        executemany_batch_page_size: int = ...,
        executemany_values_page_size: int = ...,
        **kwargs,
    ) -> None: ...
    def initialize(self, connection) -> None: ...
    @classmethod
    def dbapi(cls): ...
    def set_isolation_level(self, connection, level) -> None: ...
    def set_readonly(self, connection, value) -> None: ...
    def get_readonly(self, connection): ...
    def set_deferrable(self, connection, value) -> None: ...
    def get_deferrable(self, connection): ...
    def do_ping(self, dbapi_connection): ...
    def on_connect(self): ...
    def do_executemany(self, cursor, statement, parameters, context: Any | None = ...) -> None: ...
    def create_connect_args(self, url): ...
    def is_disconnect(self, e, connection, cursor): ...

dialect = PGDialect_psycopg2



from typing import Any

from . import fixtures

class User(fixtures.ComparableEntity): ...
class Order(fixtures.ComparableEntity): ...
class Dingaling(fixtures.ComparableEntity): ...
class EmailUser(User): ...
class Address(fixtures.ComparableEntity): ...
class Child1(fixtures.ComparableEntity): ...
class Child2(fixtures.ComparableEntity): ...
class Parent(fixtures.ComparableEntity): ...

class Screen:
    obj: Any
    parent: Any
    def __init__(self, obj, parent: Any | None = ...) -> None: ...

class Foo:
    data: str
    stuff: Any
    moredata: Any
    def __init__(self, moredata, stuff: str = ...) -> None: ...
    __hash__ = object.__hash__
    def __eq__(self, other): ...

class Bar:
    x: Any
    y: Any
    def __init__(self, x, y) -> None: ...
    __hash__ = object.__hash__
    def __eq__(self, other): ...

class OldSchool:
    x: Any
    y: Any
    def __init__(self, x, y) -> None: ...
    def __eq__(self, other): ...

class OldSchoolWithoutCompare:
    x: Any
    y: Any
    def __init__(self, x, y) -> None: ...

class BarWithoutCompare:
    x: Any
    y: Any
    def __init__(self, x, y) -> None: ...

class NotComparable:
    data: Any
    def __init__(self, data) -> None: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...

class BrokenComparable:
    data: Any
    def __init__(self, data) -> None: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from typing import Any

from . import (
    firebird as firebird,
    mssql as mssql,
    mysql as mysql,
    oracle as oracle,
    postgresql as postgresql,
    sqlite as sqlite,
    sybase as sybase,
)

__all__ = ("firebird", "mssql", "mysql", "oracle", "postgresql", "sqlite", "sybase")

registry: Any
plugins: Any



from collections.abc import Callable
from typing import Any
from typing_extensions import TypeAlias

_MypyFile: TypeAlias = Any  # mypy.nodes.MypyFile
_AttributeContext: TypeAlias = Any  # mypy.plugin.AttributeContext
_ClassDefContext: TypeAlias = Any  # mypy.plugin.ClassDefContext
_DynamicClassDefContext: TypeAlias = Any  # mypy.plugin.DynamicClassDefContext
_Plugin: TypeAlias = Any  # mypy.plugin.Plugin
_Type: TypeAlias = Any  # mypy.types.Type

class SQLAlchemyPlugin(_Plugin):
    def get_dynamic_class_hook(self, fullname: str) -> Callable[[_DynamicClassDefContext], None] | None: ...
    def get_customize_class_mro_hook(self, fullname: str) -> Callable[[_ClassDefContext], None] | None: ...
    def get_class_decorator_hook(self, fullname: str) -> Callable[[_ClassDefContext], None] | None: ...
    def get_metaclass_hook(self, fullname: str) -> Callable[[_ClassDefContext], None] | None: ...
    def get_base_class_hook(self, fullname: str) -> Callable[[_ClassDefContext], None] | None: ...
    def get_attribute_hook(self, fullname: str) -> Callable[[_AttributeContext], _Type] | None: ...
    def get_additional_deps(self, file: _MypyFile) -> list[tuple[int, str, int]]: ...

def plugin(version: str) -> type[SQLAlchemyPlugin]: ...



from typing import Any

import sqlalchemy.types as sqltypes

from .array import ARRAY as PGARRAY
from .base import ENUM, INTERVAL, UUID, PGCompiler, PGDialect, PGExecutionContext, PGIdentifierPreparer
from .json import JSON, JSONB, JSONPathType

class _PGNumeric(sqltypes.Numeric):
    def result_processor(self, dialect, coltype): ...

class _PGNumericNoBind(_PGNumeric):
    def bind_processor(self, dialect) -> None: ...

class _PGJSON(JSON):
    def result_processor(self, dialect, coltype) -> None: ...
    def get_dbapi_type(self, dbapi): ...

class _PGJSONB(JSONB):
    def result_processor(self, dialect, coltype) -> None: ...
    def get_dbapi_type(self, dbapi): ...

class _PGJSONIndexType(sqltypes.JSON.JSONIndexType):
    def get_dbapi_type(self, dbapi) -> None: ...

class _PGJSONIntIndexType(sqltypes.JSON.JSONIntIndexType):
    def get_dbapi_type(self, dbapi): ...

class _PGJSONStrIndexType(sqltypes.JSON.JSONStrIndexType):
    def get_dbapi_type(self, dbapi): ...

class _PGJSONPathType(JSONPathType):
    def get_dbapi_type(self, dbapi): ...

class _PGUUID(UUID):
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class _PGEnum(ENUM):
    def get_dbapi_type(self, dbapi): ...

class _PGInterval(INTERVAL):
    def get_dbapi_type(self, dbapi): ...
    @classmethod
    def adapt_emulated_to_native(cls, interval, **kw): ...

class _PGTimeStamp(sqltypes.DateTime):
    def get_dbapi_type(self, dbapi): ...

class _PGTime(sqltypes.Time):
    def get_dbapi_type(self, dbapi): ...

class _PGInteger(sqltypes.Integer):
    def get_dbapi_type(self, dbapi): ...

class _PGSmallInteger(sqltypes.SmallInteger):
    def get_dbapi_type(self, dbapi): ...

class _PGNullType(sqltypes.NullType):
    def get_dbapi_type(self, dbapi): ...

class _PGBigInteger(sqltypes.BigInteger):
    def get_dbapi_type(self, dbapi): ...

class _PGBoolean(sqltypes.Boolean):
    def get_dbapi_type(self, dbapi): ...

class _PGARRAY(PGARRAY):
    def bind_expression(self, bindvalue): ...

class PGExecutionContext_pg8000(PGExecutionContext):
    def create_server_side_cursor(self): ...
    def pre_exec(self) -> None: ...

class ServerSideCursor:
    server_side: bool
    ident: Any
    cursor: Any
    def __init__(self, cursor, ident) -> None: ...
    @property
    def connection(self): ...
    @property
    def rowcount(self): ...
    @property
    def description(self): ...
    def execute(self, operation, args=..., stream: Any | None = ...): ...
    def executemany(self, operation, param_sets): ...
    def fetchone(self): ...
    def fetchmany(self, num: Any | None = ...): ...
    def fetchall(self): ...
    def close(self) -> None: ...
    def setinputsizes(self, *sizes) -> None: ...
    def setoutputsize(self, size, column: Any | None = ...) -> None: ...

class PGCompiler_pg8000(PGCompiler):
    def visit_mod_binary(self, binary, operator, **kw): ...

class PGIdentifierPreparer_pg8000(PGIdentifierPreparer):
    def __init__(self, *args, **kwargs) -> None: ...

class PGDialect_pg8000(PGDialect):
    driver: str
    supports_statement_cache: bool
    supports_unicode_statements: bool
    supports_unicode_binds: bool
    default_paramstyle: str
    supports_sane_multi_rowcount: bool
    statement_compiler: Any
    preparer: Any
    supports_server_side_cursors: bool
    use_setinputsizes: bool
    description_encoding: Any
    colspecs: Any
    client_encoding: Any
    def __init__(self, client_encoding: Any | None = ..., **kwargs) -> None: ...
    @classmethod
    def dbapi(cls): ...
    def create_connect_args(self, url): ...
    def is_disconnect(self, e, connection, cursor): ...
    def set_isolation_level(self, connection, level) -> None: ...
    def set_readonly(self, connection, value) -> None: ...
    def get_readonly(self, connection): ...
    def set_deferrable(self, connection, value) -> None: ...
    def get_deferrable(self, connection): ...
    def set_client_encoding(self, connection, client_encoding) -> None: ...
    def do_set_input_sizes(self, cursor, list_of_tuples, context) -> None: ...
    def do_begin_twophase(self, connection, xid) -> None: ...
    def do_prepare_twophase(self, connection, xid) -> None: ...
    def do_rollback_twophase(self, connection, xid, is_prepared: bool = ..., recover: bool = ...) -> None: ...
    def do_commit_twophase(self, connection, xid, is_prepared: bool = ..., recover: bool = ...) -> None: ...
    def do_recover_twophase(self, connection): ...
    def on_connect(self): ...

dialect = PGDialect_pg8000



from typing import Any, Generic, TypeVar

from . import attributes, strategies
from .query import Query

_T = TypeVar("_T")

class DynaLoader(strategies.AbstractRelationshipLoader):
    logger: Any
    is_class_level: bool
    def init_class_attribute(self, mapper) -> None: ...

class DynamicAttributeImpl(attributes.AttributeImpl):
    uses_objects: bool
    default_accepts_scalar_loader: bool
    supports_population: bool
    collection: bool
    dynamic: bool
    order_by: Any
    target_mapper: Any
    query_class: Any
    def __init__(
        self, class_, key, typecallable, dispatch, target_mapper, order_by, query_class: Any | None = ..., **kw
    ) -> None: ...
    def get(self, state, dict_, passive=...): ...
    def get_collection(self, state, dict_, user_data: Any | None = ..., passive=...): ...
    def fire_append_event(self, state, dict_, value, initiator, collection_history: Any | None = ...) -> None: ...
    def fire_remove_event(self, state, dict_, value, initiator, collection_history: Any | None = ...) -> None: ...
    def set(
        self,
        state,
        dict_,
        value,
        initiator: Any | None = ...,
        passive=...,
        check_old: Any | None = ...,
        pop: bool = ...,
        _adapt: bool = ...,
    ) -> None: ...
    def delete(self, *args, **kwargs) -> None: ...
    def set_committed_value(self, state, dict_, value) -> None: ...
    def get_history(self, state, dict_, passive=...): ...
    def get_all_pending(self, state, dict_, passive=...): ...
    def append(self, state, dict_, value, initiator, passive=...) -> None: ...
    def remove(self, state, dict_, value, initiator, passive=...) -> None: ...
    def pop(self, state, dict_, value, initiator, passive=...) -> None: ...

class DynamicCollectionAdapter:
    data: Any
    def __init__(self, data) -> None: ...
    def __iter__(self): ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    def __nonzero__(self) -> bool: ...

class AppenderMixin:
    query_class: Any
    instance: Any
    attr: Any
    def __init__(self, attr, state) -> None: ...
    session: Any
    def __getitem__(self, index): ...
    def count(self): ...
    def extend(self, iterator) -> None: ...
    def append(self, item) -> None: ...
    def remove(self, item) -> None: ...

class AppenderQuery(AppenderMixin, Query[_T], Generic[_T]): ...

def mixin_user_query(cls): ...

class CollectionHistory:
    unchanged_items: Any
    added_items: Any
    deleted_items: Any
    def __init__(self, attr, state, apply_to: Any | None = ...) -> None: ...
    @property
    def added_plus_unchanged(self): ...
    @property
    def all_items(self): ...
    def as_history(self): ...
    def indexed(self, index): ...
    def add_added(self, value) -> None: ...
    def add_removed(self, value) -> None: ...



from typing import Any

class _PlainColumnGetter:
    cols: Any
    composite: Any
    def __init__(self, cols) -> None: ...
    def __reduce__(self): ...
    def __call__(self, value): ...

class _SerializableColumnGetter:
    colkeys: Any
    composite: Any
    def __init__(self, colkeys) -> None: ...
    def __reduce__(self): ...
    def __call__(self, value): ...

class _SerializableColumnGetterV2(_PlainColumnGetter):
    colkeys: Any
    composite: Any
    def __init__(self, colkeys) -> None: ...
    def __reduce__(self): ...

def column_mapped_collection(mapping_spec): ...

class _SerializableAttrGetter:
    name: Any
    getter: Any
    def __init__(self, name) -> None: ...
    def __call__(self, target): ...
    def __reduce__(self): ...

def attribute_mapped_collection(attr_name): ...
def mapped_collection(keyfunc): ...

class collection:
    @staticmethod
    def appender(fn): ...
    @staticmethod
    def remover(fn): ...
    @staticmethod
    def iterator(fn): ...
    @staticmethod
    def internally_instrumented(fn): ...
    @staticmethod
    def converter(fn): ...
    @staticmethod
    def adds(arg): ...
    @staticmethod
    def replaces(arg): ...
    @staticmethod
    def removes(arg): ...
    @staticmethod
    def removes_return(): ...

collection_adapter: Any

class CollectionAdapter:
    attr: Any
    owner_state: Any
    invalidated: bool
    empty: bool
    def __init__(self, attr, owner_state, data) -> None: ...
    @property
    def data(self): ...
    def bulk_appender(self): ...
    def append_with_event(self, item, initiator: Any | None = ...) -> None: ...
    def append_without_event(self, item) -> None: ...
    def append_multiple_without_event(self, items) -> None: ...
    def bulk_remover(self): ...
    def remove_with_event(self, item, initiator: Any | None = ...) -> None: ...
    def remove_without_event(self, item) -> None: ...
    def clear_with_event(self, initiator: Any | None = ...) -> None: ...
    def clear_without_event(self) -> None: ...
    def __iter__(self): ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    def __nonzero__(self) -> bool: ...
    def fire_append_wo_mutation_event(self, item, initiator: Any | None = ...): ...
    def fire_append_event(self, item, initiator: Any | None = ...): ...
    def fire_remove_event(self, item, initiator: Any | None = ...) -> None: ...
    def fire_pre_remove_event(self, initiator: Any | None = ...) -> None: ...

class InstrumentedList(list[Any]): ...
class InstrumentedSet(set[Any]): ...
class InstrumentedDict(dict[Any, Any]): ...

class MappedCollection(dict[Any, Any]):
    keyfunc: Any
    def __init__(self, keyfunc) -> None: ...
    def set(self, value, _sa_initiator: Any | None = ...) -> None: ...
    def remove(self, value, _sa_initiator: Any | None = ...) -> None: ...



from ..sql import ddl as ddl
from . import events as events, util as util
from .base import (
    Connection as Connection,
    Engine as Engine,
    NestedTransaction as NestedTransaction,
    RootTransaction as RootTransaction,
    Transaction as Transaction,
    TwoPhaseTransaction as TwoPhaseTransaction,
)
from .create import create_engine as create_engine, engine_from_config as engine_from_config
from .cursor import (
    BaseCursorResult as BaseCursorResult,
    BufferedColumnResultProxy as BufferedColumnResultProxy,
    BufferedColumnRow as BufferedColumnRow,
    BufferedRowResultProxy as BufferedRowResultProxy,
    CursorResult as CursorResult,
    FullyBufferedResultProxy as FullyBufferedResultProxy,
    LegacyCursorResult as LegacyCursorResult,
    ResultProxy as ResultProxy,
)
from .interfaces import (
    AdaptedConnection as AdaptedConnection,
    Compiled as Compiled,
    Connectable as Connectable,
    CreateEnginePlugin as CreateEnginePlugin,
    Dialect as Dialect,
    ExceptionContext as ExceptionContext,
    ExecutionContext as ExecutionContext,
    TypeCompiler as TypeCompiler,
)
from .mock import create_mock_engine as create_mock_engine
from .reflection import Inspector as Inspector
from .result import (
    ChunkedIteratorResult as ChunkedIteratorResult,
    FrozenResult as FrozenResult,
    IteratorResult as IteratorResult,
    MappingResult as MappingResult,
    MergedResult as MergedResult,
    Result as Result,
    ScalarResult as ScalarResult,
    result_tuple as result_tuple,
)
from .row import BaseRow as BaseRow, LegacyRow as LegacyRow, Row as Row, RowMapping as RowMapping
from .url import URL as URL, make_url as make_url
from .util import connection_memoize as connection_memoize



from typing import Any, Generic, TypeVar

from .base import SchemaEventTarget
from .operators import ColumnOperators
from .traversals import HasCacheKey
from .type_api import (
    Emulated as Emulated,
    NativeForEmulated as NativeForEmulated,
    TypeDecorator as TypeDecorator,
    TypeEngine as TypeEngine,
    Variant as Variant,
    to_instance as to_instance,
)

_T = TypeVar("_T")

class _LookupExpressionAdapter:
    class Comparator(TypeEngine.Comparator[Any]): ...
    comparator_factory: Any

class Concatenable:
    class Comparator(TypeEngine.Comparator[_T], Generic[_T]): ...
    comparator_factory: Any

class Indexable:
    class Comparator(TypeEngine.Comparator[_T], Generic[_T]):
        def __getitem__(self, index) -> ColumnOperators[_T]: ...
    comparator_factory: Any

class String(Concatenable, TypeEngine):
    __visit_name__: str
    RETURNS_UNICODE: Any
    RETURNS_BYTES: Any
    RETURNS_CONDITIONAL: Any
    RETURNS_UNKNOWN: Any
    length: Any
    collation: Any
    def __init__(
        self,
        length: Any | None = ...,
        collation: Any | None = ...,
        convert_unicode: bool = ...,
        unicode_error: Any | None = ...,
        _warn_on_bytestring: bool = ...,
        _expect_unicode: bool = ...,
    ) -> None: ...
    def literal_processor(self, dialect): ...
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...
    @property
    def python_type(self): ...
    def get_dbapi_type(self, dbapi): ...

class Text(String):
    __visit_name__: str

class Unicode(String):
    __visit_name__: str
    def __init__(self, length: Any | None = ..., **kwargs) -> None: ...

class UnicodeText(Text):
    __visit_name__: str
    def __init__(self, length: Any | None = ..., **kwargs) -> None: ...

class Integer(_LookupExpressionAdapter, TypeEngine):
    __visit_name__: str
    def get_dbapi_type(self, dbapi): ...
    @property
    def python_type(self): ...
    def literal_processor(self, dialect): ...

class SmallInteger(Integer):
    __visit_name__: str

class BigInteger(Integer):
    __visit_name__: str

class Numeric(_LookupExpressionAdapter, TypeEngine):
    __visit_name__: str
    precision: Any
    scale: Any
    decimal_return_scale: Any
    asdecimal: Any
    def __init__(
        self, precision: Any | None = ..., scale: Any | None = ..., decimal_return_scale: Any | None = ..., asdecimal: bool = ...
    ) -> None: ...
    def get_dbapi_type(self, dbapi): ...
    def literal_processor(self, dialect): ...
    @property
    def python_type(self): ...
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class Float(Numeric):
    __visit_name__: str
    scale: Any
    precision: Any
    asdecimal: Any
    decimal_return_scale: Any
    def __init__(self, precision: Any | None = ..., asdecimal: bool = ..., decimal_return_scale: Any | None = ...) -> None: ...
    def result_processor(self, dialect, coltype): ...

class DateTime(_LookupExpressionAdapter, TypeEngine):
    __visit_name__: str
    timezone: Any
    def __init__(self, timezone: bool = ...) -> None: ...
    def get_dbapi_type(self, dbapi): ...
    @property
    def python_type(self): ...

class Date(_LookupExpressionAdapter, TypeEngine):
    __visit_name__: str
    def get_dbapi_type(self, dbapi): ...
    @property
    def python_type(self): ...

class Time(_LookupExpressionAdapter, TypeEngine):
    __visit_name__: str
    timezone: Any
    def __init__(self, timezone: bool = ...) -> None: ...
    def get_dbapi_type(self, dbapi): ...
    @property
    def python_type(self): ...

class _Binary(TypeEngine):
    length: Any
    def __init__(self, length: Any | None = ...) -> None: ...
    def literal_processor(self, dialect): ...
    @property
    def python_type(self): ...
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...
    def coerce_compared_value(self, op, value): ...
    def get_dbapi_type(self, dbapi): ...

class LargeBinary(_Binary):
    __visit_name__: str
    def __init__(self, length: Any | None = ...) -> None: ...

class SchemaType(SchemaEventTarget):
    name: Any
    schema: Any
    metadata: Any
    inherit_schema: Any
    def __init__(
        self,
        name: Any | None = ...,
        schema: Any | None = ...,
        metadata: Any | None = ...,
        inherit_schema: bool = ...,
        quote: Any | None = ...,
        _create_events: bool = ...,
    ) -> None: ...
    def copy(self, **kw): ...
    def adapt(self, impltype, **kw): ...
    @property
    def bind(self): ...
    def create(self, bind: Any | None = ..., checkfirst: bool = ...) -> None: ...
    def drop(self, bind: Any | None = ..., checkfirst: bool = ...) -> None: ...

class Enum(Emulated, String, SchemaType):
    __visit_name__: str
    def __init__(self, *enums, **kw) -> None: ...
    @property
    def sort_key_function(self): ...
    @property
    def native(self): ...

    class Comparator(Concatenable.Comparator[Any]): ...
    comparator_factory: Any
    def as_generic(self, allow_nulltype: bool = ...): ...
    def adapt_to_emulated(self, impltype, **kw): ...
    def adapt(self, impltype, **kw): ...
    def literal_processor(self, dialect): ...
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...
    def copy(self, **kw): ...
    @property
    def python_type(self): ...

class PickleType(TypeDecorator):
    impl: Any
    cache_ok: bool
    protocol: Any
    pickler: Any
    comparator: Any
    def __init__(self, protocol=..., pickler: Any | None = ..., comparator: Any | None = ..., impl: Any | None = ...) -> None: ...
    def __reduce__(self): ...
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...
    def compare_values(self, x, y): ...

class Boolean(Emulated, TypeEngine, SchemaType):  # type: ignore[misc]
    __visit_name__: str
    native: bool
    create_constraint: Any
    name: Any
    def __init__(self, create_constraint: bool = ..., name: Any | None = ..., _create_events: bool = ...) -> None: ...
    @property
    def python_type(self): ...
    def literal_processor(self, dialect): ...
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class _AbstractInterval(_LookupExpressionAdapter, TypeEngine):
    def coerce_compared_value(self, op, value): ...

class Interval(Emulated, _AbstractInterval, TypeDecorator):  # type: ignore[misc]
    impl: Any
    epoch: Any
    cache_ok: bool
    native: Any
    second_precision: Any
    day_precision: Any
    def __init__(self, native: bool = ..., second_precision: Any | None = ..., day_precision: Any | None = ...) -> None: ...
    @property
    def python_type(self): ...
    def adapt_to_emulated(self, impltype, **kw): ...
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class JSON(Indexable, TypeEngine):
    __visit_name__: str
    hashable: bool
    NULL: Any
    none_as_null: Any
    def __init__(self, none_as_null: bool = ...) -> None: ...

    class JSONElementType(TypeEngine):
        def string_bind_processor(self, dialect): ...
        def string_literal_processor(self, dialect): ...
        def bind_processor(self, dialect): ...
        def literal_processor(self, dialect): ...

    class JSONIndexType(JSONElementType): ...
    class JSONIntIndexType(JSONIndexType): ...
    class JSONStrIndexType(JSONIndexType): ...
    class JSONPathType(JSONElementType): ...

    class Comparator(Indexable.Comparator[Any], Concatenable.Comparator[Any]):
        def as_boolean(self): ...
        def as_string(self): ...
        def as_integer(self): ...
        def as_float(self): ...
        def as_numeric(self, precision, scale, asdecimal: bool = ...): ...
        def as_json(self): ...
    comparator_factory: Any
    @property
    def python_type(self): ...
    @property  # type: ignore[override]
    def should_evaluate_none(self): ...
    @should_evaluate_none.setter
    def should_evaluate_none(self, value) -> None: ...
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class ARRAY(SchemaEventTarget, Indexable, Concatenable, TypeEngine):
    __visit_name__: str
    zero_indexes: bool

    class Comparator(Indexable.Comparator[_T], Concatenable.Comparator[_T], Generic[_T]):
        def contains(self, *arg, **kw) -> ColumnOperators[_T]: ...
        def any(self, other, operator: Any | None = ...): ...
        def all(self, other, operator: Any | None = ...): ...
    comparator_factory: Any
    item_type: Any
    as_tuple: Any
    dimensions: Any
    def __init__(self, item_type, as_tuple: bool = ..., dimensions: Any | None = ..., zero_indexes: bool = ...) -> None: ...
    @property
    def hashable(self): ...
    @property
    def python_type(self): ...
    def compare_values(self, x, y): ...

class TupleType(TypeEngine):
    types: Any
    def __init__(self, *types) -> None: ...
    def result_processor(self, dialect, coltype) -> None: ...

class REAL(Float):
    __visit_name__: str

class FLOAT(Float):
    __visit_name__: str

class NUMERIC(Numeric):
    __visit_name__: str

class DECIMAL(Numeric):
    __visit_name__: str

class INTEGER(Integer):
    __visit_name__: str

INT = INTEGER

class SMALLINT(SmallInteger):
    __visit_name__: str

class BIGINT(BigInteger):
    __visit_name__: str

class TIMESTAMP(DateTime):
    __visit_name__: str
    def __init__(self, timezone: bool = ...) -> None: ...
    def get_dbapi_type(self, dbapi): ...

class DATETIME(DateTime):
    __visit_name__: str

class DATE(Date):
    __visit_name__: str

class TIME(Time):
    __visit_name__: str

class TEXT(Text):
    __visit_name__: str

class CLOB(Text):
    __visit_name__: str

class VARCHAR(String):
    __visit_name__: str

class NVARCHAR(Unicode):
    __visit_name__: str

class CHAR(String):
    __visit_name__: str

class NCHAR(Unicode):
    __visit_name__: str

class BLOB(LargeBinary):
    __visit_name__: str

class BINARY(_Binary):
    __visit_name__: str

class VARBINARY(_Binary):
    __visit_name__: str

class BOOLEAN(Boolean):
    __visit_name__: str

class NullType(TypeEngine):
    __visit_name__: str
    def literal_processor(self, dialect): ...

    class Comparator(TypeEngine.Comparator[Any]): ...
    comparator_factory: Any

class TableValueType(HasCacheKey, TypeEngine):
    def __init__(self, *elements) -> None: ...

class MatchType(Boolean): ...

NULLTYPE: Any
BOOLEANTYPE: Any
STRINGTYPE: Any
INTEGERTYPE: Any
MATCHTYPE: Any
TABLEVALUE: Any



from typing import Any

from sqlalchemy import types as sqltypes
from sqlalchemy.dialects.sybase.base import SybaseDialect, SybaseExecutionContext, SybaseSQLCompiler

class _SybNumeric(sqltypes.Numeric):
    def result_processor(self, dialect, type_): ...

class SybaseExecutionContext_pysybase(SybaseExecutionContext):
    def set_ddl_autocommit(self, dbapi_connection, value) -> None: ...
    def pre_exec(self) -> None: ...

class SybaseSQLCompiler_pysybase(SybaseSQLCompiler):
    def bindparam_string(self, name, **kw): ...

class SybaseDialect_pysybase(SybaseDialect):
    driver: str
    statement_compiler: Any
    supports_statement_cache: bool
    colspecs: Any
    @classmethod
    def dbapi(cls): ...
    def create_connect_args(self, url): ...
    def do_executemany(self, cursor, statement, parameters, context: Any | None = ...) -> None: ...
    def is_disconnect(self, e, connection, cursor): ...

dialect = SybaseDialect_pysybase



from typing import Any
from typing_extensions import TypeAlias

from ...util import symbol

_ClassDef: TypeAlias = Any  # mypy.nodes.ClassDef
_Expression: TypeAlias = Any  # mypy.nodes.Expression
_MemberExpr: TypeAlias = Any  # mypy.nodes.MemberExpr
_NameExpr: TypeAlias = Any  # mypy.nodes.NameExpr
_SymbolNode: TypeAlias = Any  # mypy.nodes.SymbolNode
_TypeInfo: TypeAlias = Any  # mypy.nodes.TypeInfo
_SemanticAnalyzerPluginInterface: TypeAlias = Any  # mypy.plugin.SemanticAnalyzerPluginInterface
_UnboundType: TypeAlias = Any  # mypy.types.UnboundType

COLUMN: symbol
RELATIONSHIP: symbol
REGISTRY: symbol
COLUMN_PROPERTY: symbol
TYPEENGINE: symbol
MAPPED: symbol
DECLARATIVE_BASE: symbol
DECLARATIVE_META: symbol
MAPPED_DECORATOR: symbol
SYNONYM_PROPERTY: symbol
COMPOSITE_PROPERTY: symbol
DECLARED_ATTR: symbol
MAPPER_PROPERTY: symbol
AS_DECLARATIVE: symbol
AS_DECLARATIVE_BASE: symbol
DECLARATIVE_MIXIN: symbol
QUERY_EXPRESSION: symbol

def has_base_type_id(info: _TypeInfo, type_id: int) -> bool: ...
def mro_has_id(mro: list[_TypeInfo], type_id: int) -> bool: ...
def type_id_for_unbound_type(type_: _UnboundType, cls: _ClassDef, api: _SemanticAnalyzerPluginInterface) -> int | None: ...
def type_id_for_callee(callee: _Expression) -> int | None: ...
def type_id_for_named_node(node: _NameExpr | _MemberExpr | _SymbolNode) -> int | None: ...
def type_id_for_fullname(fullname: str) -> int | None: ...



from . import sqltypes as sqltypes
from .base import Executable as Executable
from .compiler import (
    COLLECT_CARTESIAN_PRODUCTS as COLLECT_CARTESIAN_PRODUCTS,
    FROM_LINTING as FROM_LINTING,
    NO_LINTING as NO_LINTING,
    WARN_LINTING as WARN_LINTING,
)
from .expression import (
    LABEL_STYLE_DEFAULT as LABEL_STYLE_DEFAULT,
    LABEL_STYLE_DISAMBIGUATE_ONLY as LABEL_STYLE_DISAMBIGUATE_ONLY,
    LABEL_STYLE_NONE as LABEL_STYLE_NONE,
    LABEL_STYLE_TABLENAME_PLUS_COL as LABEL_STYLE_TABLENAME_PLUS_COL,
    Alias as Alias,
    ClauseElement as ClauseElement,
    ColumnCollection as ColumnCollection,
    ColumnElement as ColumnElement,
    CompoundSelect as CompoundSelect,
    Delete as Delete,
    False_ as False_,
    FromClause as FromClause,
    Insert as Insert,
    Join as Join,
    LambdaElement as LambdaElement,
    Select as Select,
    Selectable as Selectable,
    StatementLambdaElement as StatementLambdaElement,
    Subquery as Subquery,
    TableClause as TableClause,
    TableSample as TableSample,
    True_ as True_,
    Update as Update,
    Values as Values,
    alias as alias,
    all_ as all_,
    and_ as and_,
    any_ as any_,
    asc as asc,
    between as between,
    bindparam as bindparam,
    case as case,
    cast as cast,
    collate as collate,
    column as column,
    cte as cte,
    delete as delete,
    desc as desc,
    distinct as distinct,
    except_ as except_,
    except_all as except_all,
    exists as exists,
    extract as extract,
    false as false,
    func as func,
    funcfilter as funcfilter,
    insert as insert,
    intersect as intersect,
    intersect_all as intersect_all,
    join as join,
    label as label,
    lambda_stmt as lambda_stmt,
    lateral as lateral,
    literal as literal,
    literal_column as literal_column,
    modifier as modifier,
    not_ as not_,
    null as null,
    nulls_first as nulls_first,
    nulls_last as nulls_last,
    nullsfirst as nullsfirst,
    nullslast as nullslast,
    or_ as or_,
    outerjoin as outerjoin,
    outparam as outparam,
    over as over,
    quoted_name as quoted_name,
    select as select,
    subquery as subquery,
    table as table,
    tablesample as tablesample,
    text as text,
    true as true,
    tuple_ as tuple_,
    type_coerce as type_coerce,
    union as union,
    union_all as union_all,
    update as update,
    values as values,
    within_group as within_group,
)
from .visitors import ClauseVisitor as ClauseVisitor



from ...orm.decl_api import (
    DeclarativeMeta as DeclarativeMeta,
    as_declarative as as_declarative,
    declarative_base as declarative_base,
    declared_attr as declared_attr,
    has_inherited_table as has_inherited_table,
    synonym_for as synonym_for,
)
from .extensions import (
    AbstractConcreteBase as AbstractConcreteBase,
    ConcreteBase as ConcreteBase,
    DeferredReflection as DeferredReflection,
    instrument_declarative as instrument_declarative,
)

__all__ = [
    "declarative_base",
    "synonym_for",
    "has_inherited_table",
    "instrument_declarative",
    "declared_attr",
    "as_declarative",
    "ConcreteBase",
    "AbstractConcreteBase",
    "DeclarativeMeta",
    "DeferredReflection",
]



from typing import Any

def classname_for_table(base, tablename, table): ...
def name_for_scalar_relationship(base, local_cls, referred_cls, constraint): ...
def name_for_collection_relationship(base, local_cls, referred_cls, constraint): ...
def generate_relationship(base, direction, return_fn, attrname, local_cls, referred_cls, **kw): ...

class AutomapBase:
    __abstract__: bool
    classes: Any
    @classmethod
    def prepare(
        cls,
        autoload_with: Any | None = ...,
        engine: Any | None = ...,
        reflect: bool = ...,
        schema: Any | None = ...,
        classname_for_table: Any | None = ...,
        collection_class: Any | None = ...,
        name_for_scalar_relationship: Any | None = ...,
        name_for_collection_relationship: Any | None = ...,
        generate_relationship: Any | None = ...,
        reflection_options=...,
    ) -> None: ...

def automap_base(declarative_base: Any | None = ..., **kw): ...



from _typeshed import Incomplete
from typing import Any

from ..sql.base import CacheableOptions, CompileState, Options
from ..sql.selectable import SelectState

LABEL_STYLE_LEGACY_ORM: Any

class QueryContext:
    class default_load_options(Options): ...
    load_options: Any
    execution_options: Any
    bind_arguments: Any
    compile_state: Any
    query: Any
    session: Any
    loaders_require_buffering: bool
    loaders_require_uniquing: bool
    params: Any
    create_eager_joins: Any
    propagated_loader_options: Any
    attributes: Any
    runid: Any
    partials: Any
    post_load_paths: Any
    autoflush: Any
    populate_existing: Any
    invoke_all_eagers: Any
    version_check: Any
    refresh_state: Any
    yield_per: Any
    identity_token: Any
    def __init__(
        self,
        compile_state,
        statement,
        params,
        session,
        load_options,
        execution_options: Any | None = ...,
        bind_arguments: Any | None = ...,
    ) -> None: ...

class ORMCompileState(CompileState):
    class default_compile_options(CacheableOptions): ...
    current_path: Any
    def __init__(self, *arg, **kw) -> None: ...
    @classmethod
    def create_for_statement(cls, statement_container, compiler, **kw) -> None: ...  # type: ignore[override]
    @classmethod
    def get_column_descriptions(cls, statement): ...
    @classmethod
    def orm_pre_session_exec(cls, session, statement, params, execution_options, bind_arguments, is_reentrant_invoke): ...
    @classmethod
    def orm_setup_cursor_result(cls, session, statement, params, execution_options, bind_arguments, result): ...

class ORMFromStatementCompileState(ORMCompileState):
    multi_row_eager_loaders: bool
    compound_eager_adapter: Any
    extra_criteria_entities: Any
    eager_joins: Any
    use_legacy_query_style: Any
    statement_container: Any
    requested_statement: Any
    dml_table: Any
    compile_options: Any
    statement: Any
    current_path: Any
    attributes: Any
    global_attributes: Any
    primary_columns: Any
    secondary_columns: Any
    dedupe_columns: Any
    create_eager_joins: Any
    order_by: Any
    @classmethod
    def create_for_statement(cls, statement_container, compiler, **kw): ...

class ORMSelectCompileState(ORMCompileState, SelectState):
    multi_row_eager_loaders: bool
    compound_eager_adapter: Any
    correlate: Any
    correlate_except: Any
    global_attributes: Any
    select_statement: Any
    for_statement: Any
    use_legacy_query_style: Any
    compile_options: Any
    label_style: Any
    current_path: Any
    eager_order_by: Any
    attributes: Any
    primary_columns: Any
    secondary_columns: Any
    dedupe_columns: Any
    eager_joins: Any
    extra_criteria_entities: Any
    create_eager_joins: Any
    from_clauses: Any
    @classmethod
    def create_for_statement(cls, statement, compiler, **kw): ...
    @classmethod
    def determine_last_joined_entity(cls, statement): ...
    @classmethod
    def all_selected_columns(cls, statement) -> None: ...
    @classmethod
    def get_columns_clause_froms(cls, statement): ...
    @classmethod
    def from_statement(cls, statement, from_statement): ...

class _QueryEntity:
    use_id_for_hash: bool
    @classmethod
    def to_compile_state(cls, compile_state, entities, entities_collection, is_current_entities): ...

class _MapperEntity(_QueryEntity):
    expr: Any
    mapper: Any
    entity_zero: Any
    is_aliased_class: Any
    path: Any
    selectable: Any
    def __init__(self, compile_state, entity, entities_collection, is_current_entities) -> None: ...
    supports_single_entity: bool
    use_id_for_hash: bool
    @property
    def type(self): ...
    @property
    def entity_zero_or_selectable(self): ...
    def corresponds_to(self, entity): ...
    def row_processor(self, context, result): ...
    def setup_compile_state(self, compile_state) -> None: ...

class _BundleEntity(_QueryEntity):
    bundle: Any
    expr: Any
    type: Any
    supports_single_entity: Any
    def __init__(
        self,
        compile_state,
        expr,
        entities_collection,
        is_current_entities: bool,
        setup_entities: bool = ...,
        parent_bundle: Incomplete | None = ...,
    ) -> None: ...
    @property
    def mapper(self): ...
    @property
    def entity_zero(self): ...
    def corresponds_to(self, entity): ...
    @property
    def entity_zero_or_selectable(self): ...
    def setup_compile_state(self, compile_state) -> None: ...
    def row_processor(self, context, result): ...

class _ColumnEntity(_QueryEntity):
    raw_column_index: Any
    translate_raw_column: Any
    @property
    def type(self): ...
    def row_processor(self, context, result): ...

class _RawColumnEntity(_ColumnEntity):
    entity_zero: Any
    mapper: Any
    supports_single_entity: bool
    expr: Any
    raw_column_index: Any
    translate_raw_column: Any
    column: Any
    entity_zero_or_selectable: Any
    def __init__(
        self,
        compile_state,
        column,
        entities_collection,
        raw_column_index,
        is_current_entities: bool,
        parent_bundle: Incomplete | None = ...,
    ) -> None: ...
    def corresponds_to(self, entity): ...
    def setup_compile_state(self, compile_state) -> None: ...

class _ORMColumnEntity(_ColumnEntity):
    supports_single_entity: bool
    expr: Any
    translate_raw_column: bool
    raw_column_index: Any
    entity_zero_or_selectable: Any
    entity_zero: Any
    mapper: Any
    column: Any
    def __init__(
        self,
        compile_state,
        column,
        entities_collection,
        parententity,
        raw_column_index,
        is_current_entities: bool,
        parent_bundle: Incomplete | None = ...,
    ) -> None: ...
    def corresponds_to(self, entity): ...
    def setup_compile_state(self, compile_state) -> None: ...

class _IdentityTokenEntity(_ORMColumnEntity):
    translate_raw_column: bool
    def setup_compile_state(self, compile_state) -> None: ...
    def row_processor(self, context, result): ...



from typing import Any

from ..util import memoized_property
from . import visitors
from .base import DialectKWArgs, Executable, SchemaEventTarget
from .elements import ColumnClause
from .selectable import TableClause

RETAIN_SCHEMA: Any
BLANK_SCHEMA: Any
NULL_UNSPECIFIED: Any

class SchemaItem(SchemaEventTarget, visitors.Visitable):
    __visit_name__: str
    create_drop_stringify_dialect: str
    @memoized_property
    def info(self): ...

class Table(DialectKWArgs, SchemaItem, TableClause):
    __visit_name__: str
    constraints: Any
    indexes: Any
    def __new__(cls, *args, **kw): ...
    def __init__(self, *args, **kw) -> None: ...
    @property
    def foreign_key_constraints(self): ...
    @property
    def key(self): ...
    @property
    def bind(self): ...
    def add_is_dependent_on(self, table) -> None: ...
    def append_column(self, column, replace_existing: bool = ...) -> None: ...  # type: ignore[override]
    def append_constraint(self, constraint) -> None: ...
    def exists(self, bind: Any | None = ...): ...
    def create(self, bind: Any | None = ..., checkfirst: bool = ...) -> None: ...
    def drop(self, bind: Any | None = ..., checkfirst: bool = ...) -> None: ...
    def tometadata(self, metadata, schema=..., referred_schema_fn: Any | None = ..., name: Any | None = ...): ...
    def to_metadata(self, metadata, schema=..., referred_schema_fn: Any | None = ..., name: Any | None = ...): ...

class Column(DialectKWArgs, SchemaItem, ColumnClause):
    __visit_name__: str
    inherit_cache: bool
    key: Any
    primary_key: Any
    nullable: Any
    default: Any
    server_default: Any
    server_onupdate: Any
    index: Any
    unique: Any
    system: Any
    doc: Any
    onupdate: Any
    autoincrement: Any
    constraints: Any
    foreign_keys: Any
    comment: Any
    computed: Any
    identity: Any
    info: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def references(self, column): ...
    def append_foreign_key(self, fk) -> None: ...
    def copy(self, **kw): ...

class ForeignKey(DialectKWArgs, SchemaItem):
    __visit_name__: str
    constraint: Any
    parent: Any
    use_alter: Any
    name: Any
    onupdate: Any
    ondelete: Any
    deferrable: Any
    initially: Any
    link_to_name: Any
    match: Any
    info: Any
    def __init__(
        self,
        column,
        _constraint: Any | None = ...,
        use_alter: bool = ...,
        name: Any | None = ...,
        onupdate: Any | None = ...,
        ondelete: Any | None = ...,
        deferrable: Any | None = ...,
        initially: Any | None = ...,
        link_to_name: bool = ...,
        match: Any | None = ...,
        info: Any | None = ...,
        _unresolvable: bool = ...,
        **dialect_kw,
    ) -> None: ...
    def copy(self, schema: Any | None = ..., **kw): ...
    @property
    def target_fullname(self): ...
    def references(self, table): ...
    def get_referent(self, table): ...
    @memoized_property
    def column(self): ...

class DefaultGenerator(Executable, SchemaItem):
    __visit_name__: str
    is_sequence: bool
    is_server_default: bool
    column: Any
    for_update: Any
    def __init__(self, for_update: bool = ...) -> None: ...
    def execute(self, bind: Any | None = ...): ...  # type: ignore[override]
    @property
    def bind(self): ...

class ColumnDefault(DefaultGenerator):
    arg: Any
    def __init__(self, arg, **kwargs) -> None: ...
    @memoized_property
    def is_callable(self): ...
    @memoized_property
    def is_clause_element(self): ...
    @memoized_property
    def is_scalar(self): ...

class IdentityOptions:
    start: Any
    increment: Any
    minvalue: Any
    maxvalue: Any
    nominvalue: Any
    nomaxvalue: Any
    cycle: Any
    cache: Any
    order: Any
    def __init__(
        self,
        start: Any | None = ...,
        increment: Any | None = ...,
        minvalue: Any | None = ...,
        maxvalue: Any | None = ...,
        nominvalue: Any | None = ...,
        nomaxvalue: Any | None = ...,
        cycle: Any | None = ...,
        cache: Any | None = ...,
        order: Any | None = ...,
    ) -> None: ...

class Sequence(IdentityOptions, DefaultGenerator):
    __visit_name__: str
    is_sequence: bool
    name: Any
    optional: Any
    schema: Any
    metadata: Any
    data_type: Any
    def __init__(
        self,
        name,
        start: Any | None = ...,
        increment: Any | None = ...,
        minvalue: Any | None = ...,
        maxvalue: Any | None = ...,
        nominvalue: Any | None = ...,
        nomaxvalue: Any | None = ...,
        cycle: Any | None = ...,
        schema: Any | None = ...,
        cache: Any | None = ...,
        order: Any | None = ...,
        data_type: Any | None = ...,
        optional: bool = ...,
        quote: Any | None = ...,
        metadata: Any | None = ...,
        quote_schema: Any | None = ...,
        for_update: bool = ...,
    ) -> None: ...
    @memoized_property
    def is_callable(self): ...
    @memoized_property
    def is_clause_element(self): ...
    def next_value(self): ...
    @property
    def bind(self): ...
    def create(self, bind: Any | None = ..., checkfirst: bool = ...) -> None: ...
    def drop(self, bind: Any | None = ..., checkfirst: bool = ...) -> None: ...

class FetchedValue(SchemaEventTarget):
    is_server_default: bool
    reflected: bool
    has_argument: bool
    is_clause_element: bool
    for_update: Any
    def __init__(self, for_update: bool = ...) -> None: ...

class DefaultClause(FetchedValue):
    has_argument: bool
    arg: Any
    reflected: Any
    def __init__(self, arg, for_update: bool = ..., _reflected: bool = ...) -> None: ...

class Constraint(DialectKWArgs, SchemaItem):
    __visit_name__: str
    name: Any
    deferrable: Any
    initially: Any
    info: Any
    def __init__(
        self,
        name: Any | None = ...,
        deferrable: Any | None = ...,
        initially: Any | None = ...,
        _create_rule: Any | None = ...,
        info: Any | None = ...,
        _type_bound: bool = ...,
        **dialect_kw,
    ) -> None: ...
    @property
    def table(self): ...
    def copy(self, **kw): ...

class ColumnCollectionMixin:
    columns: Any
    def __init__(self, *columns, **kw) -> None: ...

class ColumnCollectionConstraint(ColumnCollectionMixin, Constraint):
    def __init__(self, *columns, **kw) -> None: ...
    columns: Any
    def __contains__(self, x): ...
    def copy(self, target_table: Any | None = ..., **kw): ...
    def contains_column(self, col): ...
    def __iter__(self): ...
    def __len__(self) -> int: ...

class CheckConstraint(ColumnCollectionConstraint):
    __visit_name__: str
    sqltext: Any
    def __init__(
        self,
        sqltext,
        name: Any | None = ...,
        deferrable: Any | None = ...,
        initially: Any | None = ...,
        table: Any | None = ...,
        info: Any | None = ...,
        _create_rule: Any | None = ...,
        _autoattach: bool = ...,
        _type_bound: bool = ...,
        **kw,
    ) -> None: ...
    @property
    def is_column_level(self): ...
    def copy(self, target_table: Any | None = ..., **kw): ...

class ForeignKeyConstraint(ColumnCollectionConstraint):
    __visit_name__: str
    onupdate: Any
    ondelete: Any
    link_to_name: Any
    use_alter: Any
    match: Any
    elements: Any
    def __init__(
        self,
        columns,
        refcolumns,
        name: Any | None = ...,
        onupdate: Any | None = ...,
        ondelete: Any | None = ...,
        deferrable: Any | None = ...,
        initially: Any | None = ...,
        use_alter: bool = ...,
        link_to_name: bool = ...,
        match: Any | None = ...,
        table: Any | None = ...,
        info: Any | None = ...,
        **dialect_kw,
    ) -> None: ...
    columns: Any
    @property
    def referred_table(self): ...
    @property
    def column_keys(self): ...
    def copy(self, schema: Any | None = ..., target_table: Any | None = ..., **kw): ...  # type: ignore[override]

class PrimaryKeyConstraint(ColumnCollectionConstraint):
    __visit_name__: str
    def __init__(self, *columns, **kw) -> None: ...
    @property
    def columns_autoinc_first(self): ...

class UniqueConstraint(ColumnCollectionConstraint):
    __visit_name__: str

class Index(DialectKWArgs, ColumnCollectionMixin, SchemaItem):
    __visit_name__: str
    table: Any
    name: Any
    unique: Any
    info: Any
    expressions: Any
    def __init__(self, name, *expressions, **kw) -> None: ...
    @property
    def bind(self): ...
    def create(self, bind: Any | None = ..., checkfirst: bool = ...): ...
    def drop(self, bind: Any | None = ..., checkfirst: bool = ...) -> None: ...

DEFAULT_NAMING_CONVENTION: Any

class MetaData(SchemaItem):
    __visit_name__: str
    tables: Any
    schema: Any
    naming_convention: Any
    info: Any
    def __init__(
        self,
        bind: Any | None = ...,
        schema: Any | None = ...,
        quote_schema: Any | None = ...,
        naming_convention: Any | None = ...,
        info: Any | None = ...,
    ) -> None: ...
    def __contains__(self, table_or_key) -> bool: ...
    def is_bound(self): ...
    bind: Any
    def clear(self) -> None: ...
    def remove(self, table) -> None: ...
    @property
    def sorted_tables(self): ...
    def reflect(
        self,
        bind: Any | None = ...,
        schema: Any | None = ...,
        views: bool = ...,
        only: Any | None = ...,
        extend_existing: bool = ...,
        autoload_replace: bool = ...,
        resolve_fks: bool = ...,
        **dialect_kwargs,
    ) -> None: ...
    def create_all(self, bind: Any | None = ..., tables: Any | None = ..., checkfirst: bool = ...) -> None: ...
    def drop_all(self, bind: Any | None = ..., tables: Any | None = ..., checkfirst: bool = ...) -> None: ...

class ThreadLocalMetaData(MetaData):
    __visit_name__: str
    context: Any
    def __init__(self) -> None: ...
    bind: Any
    def is_bound(self): ...
    def dispose(self) -> None: ...

class Computed(FetchedValue, SchemaItem):
    __visit_name__: str
    sqltext: Any
    persisted: Any
    column: Any
    def __init__(self, sqltext, persisted: Any | None = ...) -> None: ...
    def copy(self, target_table: Any | None = ..., **kw): ...

class Identity(IdentityOptions, FetchedValue, SchemaItem):
    __visit_name__: str
    always: Any
    on_null: Any
    column: Any
    def __init__(
        self,
        always: bool = ...,
        on_null: Any | None = ...,
        start: Any | None = ...,
        increment: Any | None = ...,
        minvalue: Any | None = ...,
        maxvalue: Any | None = ...,
        nominvalue: Any | None = ...,
        nomaxvalue: Any | None = ...,
        cycle: Any | None = ...,
        cache: Any | None = ...,
        order: Any | None = ...,
    ) -> None: ...
    def copy(self, **kw): ...



from typing import Any

from ..sql import operators

class UnevaluatableError(Exception): ...

class _NoObject(operators.ColumnOperators[Any]):
    def operate(self, *arg, **kw) -> None: ...
    def reverse_operate(self, *arg, **kw) -> None: ...

class EvaluatorCompiler:
    target_cls: Any
    def __init__(self, target_cls: Any | None = ...) -> None: ...
    def process(self, *clauses): ...
    def visit_grouping(self, clause): ...
    def visit_null(self, clause): ...
    def visit_false(self, clause): ...
    def visit_true(self, clause): ...
    def visit_column(self, clause): ...
    def visit_tuple(self, clause): ...
    def visit_clauselist(self, clause): ...
    def visit_binary(self, clause): ...
    def visit_unary(self, clause): ...
    def visit_bindparam(self, clause): ...



from typing import Any

class BaseRow:
    def __init__(self, parent, processors, keymap, key_style, data) -> None: ...
    def __reduce__(self): ...
    def __iter__(self): ...
    def __len__(self) -> int: ...
    def __hash__(self) -> int: ...
    __getitem__: Any

def safe_rowproxy_reconstructor(__cls, __state): ...



from typing import Any

from ...sql import expression
from ...sql.schema import ColumnCollectionConstraint

class aggregate_order_by(expression.ColumnElement[Any]):
    __visit_name__: str
    stringify_dialect: str
    target: Any
    type: Any
    order_by: Any
    def __init__(self, target, *order_by) -> None: ...
    def self_group(self, against: Any | None = ...): ...
    def get_children(self, **kwargs): ...

class ExcludeConstraint(ColumnCollectionConstraint):
    __visit_name__: str
    where: Any
    inherit_cache: bool
    create_drop_stringify_dialect: str
    operators: Any
    using: Any
    ops: Any
    def __init__(self, *elements, **kw) -> None: ...

def array_agg(*arg, **kw): ...



import abc

class ReversibleProxy: ...

class StartableContext(abc.ABC, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    async def start(self, is_ctxmanager: bool = ...): ...
    def __await__(self): ...
    async def __aenter__(self): ...
    @abc.abstractmethod
    async def __aexit__(self, type_, value, traceback): ...

class ProxyComparable(ReversibleProxy):
    def __hash__(self) -> int: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from typing import Any

from ...engine import default
from ...sql import compiler
from ...types import BINARY as BINARY, BLOB as BLOB, BOOLEAN as BOOLEAN, DATE as DATE, VARBINARY as VARBINARY
from .enumerated import ENUM as ENUM, SET as SET
from .json import JSON as JSON
from .types import (
    BIGINT as BIGINT,
    BIT as BIT,
    CHAR as CHAR,
    DATETIME as DATETIME,
    DECIMAL as DECIMAL,
    DOUBLE as DOUBLE,
    FLOAT as FLOAT,
    INTEGER as INTEGER,
    LONGBLOB as LONGBLOB,
    LONGTEXT as LONGTEXT,
    MEDIUMBLOB as MEDIUMBLOB,
    MEDIUMINT as MEDIUMINT,
    MEDIUMTEXT as MEDIUMTEXT,
    NCHAR as NCHAR,
    NUMERIC as NUMERIC,
    NVARCHAR as NVARCHAR,
    REAL as REAL,
    SMALLINT as SMALLINT,
    TEXT as TEXT,
    TIME as TIME,
    TIMESTAMP as TIMESTAMP,
    TINYBLOB as TINYBLOB,
    TINYINT as TINYINT,
    TINYTEXT as TINYTEXT,
    VARCHAR as VARCHAR,
    YEAR as YEAR,
)

AUTOCOMMIT_RE: Any
SET_RE: Any
MSTime = TIME
MSSet = SET
MSEnum = ENUM
MSLongBlob = LONGBLOB
MSMediumBlob = MEDIUMBLOB
MSTinyBlob = TINYBLOB
MSBlob = BLOB
MSBinary = BINARY
MSVarBinary = VARBINARY
MSNChar = NCHAR
MSNVarChar = NVARCHAR
MSChar = CHAR
MSString = VARCHAR
MSLongText = LONGTEXT
MSMediumText = MEDIUMTEXT
MSTinyText = TINYTEXT
MSText = TEXT
MSYear = YEAR
MSTimeStamp = TIMESTAMP
MSBit = BIT
MSSmallInteger = SMALLINT
MSTinyInteger = TINYINT
MSMediumInteger = MEDIUMINT
MSBigInteger = BIGINT
MSNumeric = NUMERIC
MSDecimal = DECIMAL
MSDouble = DOUBLE
MSReal = REAL
MSFloat = FLOAT
MSInteger = INTEGER
colspecs: Any
ischema_names: Any

class MySQLExecutionContext(default.DefaultExecutionContext):
    def should_autocommit_text(self, statement): ...
    def create_server_side_cursor(self): ...
    def fire_sequence(self, seq, type_): ...

class MySQLCompiler(compiler.SQLCompiler):
    render_table_with_column_in_update_from: bool
    extract_map: Any
    def default_from(self): ...
    def visit_random_func(self, fn, **kw): ...
    def visit_sequence(self, seq, **kw): ...
    def visit_sysdate_func(self, fn, **kw): ...
    def visit_json_getitem_op_binary(self, binary, operator, **kw): ...
    def visit_json_path_getitem_op_binary(self, binary, operator, **kw): ...
    def visit_on_duplicate_key_update(self, on_duplicate, **kw): ...
    def visit_concat_op_binary(self, binary, operator, **kw): ...
    def visit_mysql_match(self, element, **kw): ...
    def visit_match_op_binary(self, binary, operator, **kw): ...
    def get_from_hint_text(self, table, text): ...
    def visit_typeclause(self, typeclause, type_: Any | None = ..., **kw): ...
    def visit_cast(self, cast, **kw): ...
    def render_literal_value(self, value, type_): ...
    def visit_true(self, element, **kw): ...
    def visit_false(self, element, **kw): ...
    def get_select_precolumns(self, select, **kw): ...
    def visit_join(self, join, asfrom: bool = ..., from_linter: Any | None = ..., **kwargs): ...
    def for_update_clause(self, select, **kw): ...
    def limit_clause(self, select, **kw): ...
    def update_limit_clause(self, update_stmt): ...
    def update_tables_clause(self, update_stmt, from_table, extra_froms, **kw): ...
    def update_from_clause(self, update_stmt, from_table, extra_froms, from_hints, **kw) -> None: ...
    def delete_table_clause(self, delete_stmt, from_table, extra_froms): ...
    def delete_extra_from_clause(self, delete_stmt, from_table, extra_froms, from_hints, **kw): ...
    def visit_empty_set_expr(self, element_types): ...
    def visit_is_distinct_from_binary(self, binary, operator, **kw): ...
    def visit_is_not_distinct_from_binary(self, binary, operator, **kw): ...
    def visit_regexp_match_op_binary(self, binary, operator, **kw): ...
    def visit_not_regexp_match_op_binary(self, binary, operator, **kw): ...
    def visit_regexp_replace_op_binary(self, binary, operator, **kw): ...

class MySQLDDLCompiler(compiler.DDLCompiler):
    def get_column_specification(self, column, **kw): ...
    def post_create_table(self, table): ...
    def visit_create_index(self, create, **kw): ...
    def visit_primary_key_constraint(self, constraint): ...
    def visit_drop_index(self, drop): ...
    def visit_drop_constraint(self, drop): ...
    def define_constraint_match(self, constraint): ...
    def visit_set_table_comment(self, create): ...
    def visit_drop_table_comment(self, create): ...
    def visit_set_column_comment(self, create): ...

class MySQLTypeCompiler(compiler.GenericTypeCompiler):
    def visit_NUMERIC(self, type_, **kw): ...
    def visit_DECIMAL(self, type_, **kw): ...
    def visit_DOUBLE(self, type_, **kw): ...
    def visit_REAL(self, type_, **kw): ...
    def visit_FLOAT(self, type_, **kw): ...
    def visit_INTEGER(self, type_, **kw): ...
    def visit_BIGINT(self, type_, **kw): ...
    def visit_MEDIUMINT(self, type_, **kw): ...
    def visit_TINYINT(self, type_, **kw): ...
    def visit_SMALLINT(self, type_, **kw): ...
    def visit_BIT(self, type_, **kw): ...
    def visit_DATETIME(self, type_, **kw): ...
    def visit_DATE(self, type_, **kw): ...
    def visit_TIME(self, type_, **kw): ...
    def visit_TIMESTAMP(self, type_, **kw): ...
    def visit_YEAR(self, type_, **kw): ...
    def visit_TEXT(self, type_, **kw): ...
    def visit_TINYTEXT(self, type_, **kw): ...
    def visit_MEDIUMTEXT(self, type_, **kw): ...
    def visit_LONGTEXT(self, type_, **kw): ...
    def visit_VARCHAR(self, type_, **kw): ...
    def visit_CHAR(self, type_, **kw): ...
    def visit_NVARCHAR(self, type_, **kw): ...
    def visit_NCHAR(self, type_, **kw): ...
    def visit_VARBINARY(self, type_, **kw): ...
    def visit_JSON(self, type_, **kw): ...
    def visit_large_binary(self, type_, **kw): ...
    def visit_enum(self, type_, **kw): ...
    def visit_BLOB(self, type_, **kw): ...
    def visit_TINYBLOB(self, type_, **kw): ...
    def visit_MEDIUMBLOB(self, type_, **kw): ...
    def visit_LONGBLOB(self, type_, **kw): ...
    def visit_ENUM(self, type_, **kw): ...
    def visit_SET(self, type_, **kw): ...
    def visit_BOOLEAN(self, type_, **kw): ...

class MySQLIdentifierPreparer(compiler.IdentifierPreparer):
    reserved_words: Any
    def __init__(self, dialect, server_ansiquotes: bool = ..., **kw) -> None: ...

class MariaDBIdentifierPreparer(MySQLIdentifierPreparer):
    reserved_words: Any

class MySQLDialect(default.DefaultDialect):
    logger: Any
    name: str
    supports_statement_cache: bool
    supports_alter: bool
    supports_native_boolean: bool
    max_identifier_length: int
    max_index_name_length: int
    max_constraint_name_length: int
    supports_native_enum: bool
    supports_sequences: bool
    sequences_optional: bool
    supports_for_update_of: bool
    supports_default_values: bool
    supports_default_metavalue: bool
    supports_sane_rowcount: bool
    supports_sane_multi_rowcount: bool
    supports_multivalues_insert: bool
    supports_comments: bool
    inline_comments: bool
    default_paramstyle: str
    colspecs: Any
    cte_follows_insert: bool
    statement_compiler: Any
    ddl_compiler: Any
    type_compiler: Any
    ischema_names: Any
    preparer: Any
    is_mariadb: bool
    construct_arguments: Any
    isolation_level: Any
    def __init__(
        self,
        isolation_level: Any | None = ...,
        json_serializer: Any | None = ...,
        json_deserializer: Any | None = ...,
        is_mariadb: Any | None = ...,
        **kwargs,
    ) -> None: ...
    def on_connect(self): ...
    def set_isolation_level(self, connection, level) -> None: ...
    def get_isolation_level(self, connection): ...
    def do_begin_twophase(self, connection, xid) -> None: ...
    def do_prepare_twophase(self, connection, xid) -> None: ...
    def do_rollback_twophase(self, connection, xid, is_prepared: bool = ..., recover: bool = ...) -> None: ...
    def do_commit_twophase(self, connection, xid, is_prepared: bool = ..., recover: bool = ...) -> None: ...
    def do_recover_twophase(self, connection): ...
    def is_disconnect(self, e, connection, cursor): ...
    def has_table(self, connection, table_name, schema: Any | None = ...): ...  # type: ignore[override]
    def has_sequence(self, connection, sequence_name, schema: Any | None = ...): ...  # type: ignore[override]
    def get_sequence_names(self, connection, schema: Any | None = ..., **kw): ...
    identifier_preparer: Any
    def initialize(self, connection) -> None: ...
    def get_schema_names(self, connection, **kw): ...
    def get_table_names(self, connection, schema: Any | None = ..., **kw): ...
    def get_view_names(self, connection, schema: Any | None = ..., **kw): ...
    def get_table_options(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_columns(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_pk_constraint(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_foreign_keys(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_check_constraints(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_table_comment(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_indexes(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_unique_constraints(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_view_definition(self, connection, view_name, schema: Any | None = ..., **kw): ...

class _DecodingRow:
    rowproxy: Any
    charset: Any
    def __init__(self, rowproxy, charset) -> None: ...
    def __getitem__(self, index): ...
    def __getattr__(self, attr: str): ...



from typing import Any

from ...connectors.mxodbc import MxODBCConnector
from .base import VARBINARY, MSDialect, _MSDate, _MSTime
from .pyodbc import MSExecutionContext_pyodbc, _MSNumeric_pyodbc

class _MSNumeric_mxodbc(_MSNumeric_pyodbc): ...

class _MSDate_mxodbc(_MSDate):
    def bind_processor(self, dialect): ...

class _MSTime_mxodbc(_MSTime):
    def bind_processor(self, dialect): ...

class _VARBINARY_mxodbc(VARBINARY):
    def bind_processor(self, dialect): ...

class MSExecutionContext_mxodbc(MSExecutionContext_pyodbc): ...

class MSDialect_mxodbc(MxODBCConnector, MSDialect):
    supports_statement_cache: bool
    colspecs: Any
    description_encoding: Any
    def __init__(self, description_encoding: Any | None = ..., **params) -> None: ...

dialect = MSDialect_mxodbc



from collections.abc import Sequence
from typing import Any
from typing_extensions import TypeAlias

_AssignmentStmt: TypeAlias = Any  # mypy.nodes.AssignmentStmt
_Expression: TypeAlias = Any  # mypy.nodes.Expression
_RefExpr: TypeAlias = Any  # mypy.nodes.RefExpr
_TypeInfo: TypeAlias = Any  # mypy.nodes.TypeInfo
_Var: TypeAlias = Any  # mypy.nodes.Var
_SemanticAnalyzerPluginInterface: TypeAlias = Any  # mypy.plugin.SemanticAnalyzerPluginInterface
_ProperType: TypeAlias = Any  # mypy.types.ProperType

def infer_type_from_right_hand_nameexpr(
    api: _SemanticAnalyzerPluginInterface,
    stmt: _AssignmentStmt,
    node: _Var,
    left_hand_explicit_type: _ProperType | None,
    infer_from_right_side: _RefExpr,
) -> _ProperType | None: ...
def infer_type_from_left_hand_type_only(
    api: _SemanticAnalyzerPluginInterface, node: _Var, left_hand_explicit_type: _ProperType | None
) -> _ProperType | None: ...
def extract_python_type_from_typeengine(
    api: _SemanticAnalyzerPluginInterface, node: _TypeInfo, type_args: Sequence[_Expression]
) -> _ProperType: ...



from typing import Any, ClassVar

class HasDescriptionCode:
    code: str | None
    def __init__(self, *arg: Any, code: str | None = ..., **kw: Any) -> None: ...

class SQLAlchemyError(HasDescriptionCode, Exception):
    def __unicode__(self) -> str: ...

class ArgumentError(SQLAlchemyError): ...

class ObjectNotExecutableError(ArgumentError):
    target: Any
    def __init__(self, target) -> None: ...
    def __reduce__(self): ...

class NoSuchModuleError(ArgumentError): ...
class NoForeignKeysError(ArgumentError): ...
class AmbiguousForeignKeysError(ArgumentError): ...

class CircularDependencyError(SQLAlchemyError):
    cycles: Any
    edges: Any
    def __init__(self, message, cycles, edges, msg: Any | None = ..., code: Any | None = ...) -> None: ...
    def __reduce__(self): ...

class CompileError(SQLAlchemyError): ...

class UnsupportedCompilationError(CompileError):
    code: str
    compiler: Any
    element_type: Any
    message: str | None
    def __init__(self, compiler, element_type, message: str | None = ...) -> None: ...
    def __reduce__(self): ...

class IdentifierError(SQLAlchemyError): ...

class DisconnectionError(SQLAlchemyError):
    invalidate_pool: bool

class InvalidatePoolError(DisconnectionError):
    invalidate_pool: bool

class TimeoutError(SQLAlchemyError): ...
class InvalidRequestError(SQLAlchemyError): ...
class NoInspectionAvailable(InvalidRequestError): ...
class PendingRollbackError(InvalidRequestError): ...
class ResourceClosedError(InvalidRequestError): ...
class NoSuchColumnError(InvalidRequestError, KeyError): ...
class NoResultFound(InvalidRequestError): ...
class MultipleResultsFound(InvalidRequestError): ...
class NoReferenceError(InvalidRequestError): ...

class AwaitRequired(InvalidRequestError):
    code: str

class MissingGreenlet(InvalidRequestError):
    code: str

class NoReferencedTableError(NoReferenceError):
    table_name: Any
    def __init__(self, message, tname) -> None: ...
    def __reduce__(self): ...

class NoReferencedColumnError(NoReferenceError):
    table_name: Any
    column_name: Any
    def __init__(self, message, tname, cname) -> None: ...
    def __reduce__(self): ...

class NoSuchTableError(InvalidRequestError): ...
class UnreflectableTableError(InvalidRequestError): ...
class UnboundExecutionError(InvalidRequestError): ...
class DontWrapMixin: ...

class StatementError(SQLAlchemyError):
    statement: Any
    params: Any
    orig: Any
    ismulti: Any
    hide_parameters: Any
    detail: Any
    def __init__(
        self, message, statement, params, orig, hide_parameters: bool = ..., code: Any | None = ..., ismulti: Any | None = ...
    ) -> None: ...
    def add_detail(self, msg) -> None: ...
    def __reduce__(self): ...

class DBAPIError(StatementError):
    code: str
    @classmethod
    def instance(
        cls,
        statement,
        params,
        orig,
        dbapi_base_err,
        hide_parameters: bool = ...,
        connection_invalidated: bool = ...,
        dialect: Any | None = ...,
        ismulti: Any | None = ...,
    ): ...
    def __reduce__(self): ...
    connection_invalidated: Any
    def __init__(
        self,
        statement,
        params,
        orig,
        hide_parameters: bool = ...,
        connection_invalidated: bool = ...,
        code: Any | None = ...,
        ismulti: Any | None = ...,
    ) -> None: ...

class InterfaceError(DBAPIError): ...
class DatabaseError(DBAPIError): ...
class DataError(DatabaseError): ...
class OperationalError(DatabaseError): ...
class IntegrityError(DatabaseError): ...
class InternalError(DatabaseError): ...
class ProgrammingError(DatabaseError): ...
class NotSupportedError(DatabaseError): ...

class SADeprecationWarning(HasDescriptionCode, DeprecationWarning):
    deprecated_since: ClassVar[str | None]

class Base20DeprecationWarning(SADeprecationWarning):
    deprecated_since: ClassVar[str]

class LegacyAPIWarning(Base20DeprecationWarning): ...
class RemovedIn20Warning(Base20DeprecationWarning): ...
class MovedIn20Warning(RemovedIn20Warning): ...

class SAPendingDeprecationWarning(PendingDeprecationWarning):
    deprecated_since: ClassVar[str | None]

class SAWarning(HasDescriptionCode, RuntimeWarning): ...



from typing import Any

def Table(*args, **kw): ...
def Column(*args, **kw): ...

class eq_type_affinity:
    target: Any
    def __init__(self, target) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...

class eq_clause_element:
    target: Any
    def __init__(self, target) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
